#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.association.interfaces import IAssociationParagraph, ASSOCIATION_PARAGRAPH_TYPE
from pyams_content.component.association.zmi.interfaces import IAssociationsParentForm
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget, IParagraphContainer, \
    IParagraphRenderer
from pyams_content.component.paragraph.zmi.interfaces import IParagraphInnerEditor, IParagraphContainerView
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_content.shared.common.interfaces import IWfSharedContent
from pyams_form.interfaces.form import IInnerForm, IEditFormButtons
from pyams_i18n.interfaces import II18n
from pyams_skin.interfaces.viewlet import IToolbarAddingMenu
from pyams_skin.layer import IPyAMSLayer
from z3c.form.interfaces import INPUT_MODE

# import packages
from pyams_content.component.association.paragraph import AssociationParagraph
from pyams_content.component.paragraph.zmi import BaseParagraphAJAXAddForm, BaseParagraphAJAXEditForm, \
    BaseParagraphAddMenu, BaseParagraphPropertiesEditForm
from pyams_content.features.renderer.zmi import BaseRenderedContentRenderer
from pyams_content.features.renderer.zmi.widget import RendererFieldWidget
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.event import get_json_widget_refresh_event
from pyams_utils.adapter import adapter_config
from pyams_utils.traversing import get_parent
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogAddForm
from pyramid.view import view_config
from z3c.form import field, button
from zope.interface import implementer

from pyams_content import _


@viewlet_config(name='add-association-paragraph.menu', context=IParagraphContainerTarget, view=IParagraphContainerView,
                layer=IPyAMSLayer, manager=IToolbarAddingMenu, weight=90)
class AssociationParagraphAddMenu(BaseParagraphAddMenu):
    """Associations paragraph add menu"""

    label = _("Associations...")
    label_css_class = 'fa fa-fw fa-link'
    url = 'add-association-paragraph.html'
    paragraph_type = ASSOCIATION_PARAGRAPH_TYPE


@pagelet_config(name='add-association-paragraph.html', context=IParagraphContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class AssociationParagraphAddForm(AdminDialogAddForm):
    """Association paragraph add form"""

    legend = _("Add new association paragraph")
    icon_css_class = 'fa fa-fw fa-link'

    fields = field.Fields(IAssociationParagraph).select('title', 'renderer')
    ajax_handler = 'add-association-paragraph.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def create(self, data):
        return AssociationParagraph()

    def add(self, object):
        IParagraphContainer(self.context).append(object)


@view_config(name='add-association-paragraph.json', context=IParagraphContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class AssociationParagraphAJAXAddForm(BaseParagraphAJAXAddForm, AssociationParagraphAddForm):
    """Association paragraph add form, JSON renderer"""


@pagelet_config(name='properties.html', context=IAssociationParagraph, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class AssociationParagraphPropertiesEditForm(BaseParagraphPropertiesEditForm):
    """Association paragraph properties edit form"""

    prefix = 'association_properties.'

    @property
    def title(self):
        content = get_parent(self.context, IWfSharedContent)
        return II18n(content).query_attribute('title', request=self.request)

    legend = _("Edit association paragraph properties")
    icon_css_class = 'fa fa-fw fa-link'

    fields = field.Fields(IAssociationParagraph).select('title', 'renderer')
    fields['renderer'].widgetFactory = RendererFieldWidget

    ajax_handler = 'properties.json'
    edit_permission = MANAGE_CONTENT_PERMISSION


@view_config(name='properties.json', context=IAssociationParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class AssociationParagraphPropertiesAJAXEditForm(BaseParagraphAJAXEditForm, AssociationParagraphPropertiesEditForm):
    """Association paragraph properties edit form, JSON renderer"""


@adapter_config(context=(IAssociationParagraph, IPyAMSLayer), provides=IParagraphInnerEditor)
@implementer(IInnerForm, IAssociationsParentForm)
class AssociationParagraphInnerEditForm(AssociationParagraphPropertiesEditForm):
    """Association paragraph inner edit form"""

    legend = None
    ajax_handler = 'inner-properties.json'

    @property
    def buttons(self):
        if self.mode == INPUT_MODE:
            return button.Buttons(IEditFormButtons)
        else:
            return button.Buttons()


@view_config(name='inner-properties.json', context=IAssociationParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class AssociationParagraphInnerAJAXEditForm(BaseParagraphAJAXEditForm, AssociationParagraphInnerEditForm):
    """Associations paragraph inner edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        output = super(AssociationParagraphInnerAJAXEditForm, self).get_ajax_output(changes)
        updated = changes.get(IAssociationParagraph, ())
        if 'renderer' in updated:
            output.setdefault('events', []).append(get_json_widget_refresh_event(self.context, self.request,
                                                                                 AssociationParagraphInnerEditForm,
                                                                                 'renderer'))
        return output


#
# Association paragraph renderer
#

@adapter_config(context=(IAssociationParagraph, IPyAMSLayer), provides=IParagraphRenderer)
class AssociationParagraphRenderer(BaseRenderedContentRenderer):
    """Association paragraph renderer"""
