#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
from pyams_content.component.paragraph.zmi import get_json_paragraph_markers_refresh_event

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.association.interfaces import IAssociationContainer
from pyams_content.component.association.zmi.interfaces import IAssociationsView
from pyams_content.component.links.interfaces import ILinkContainerTarget, IInternalLink, IBaseLink, \
    IExternalLink, IMailtoLink
from pyams_content.component.paragraph.zmi.interfaces import IParagraphContainerTable, IParagraphTitleToolbar
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_skin.interfaces.viewlet import IToolbarAddingMenu
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.interfaces import VIEW_SYSTEM_PERMISSION

# import packages
from pyams_content.component.association.zmi import AssociationItemAJAXAddForm, AssociationItemAJAXEditForm
from pyams_content.component.links import InternalLink, ExternalLink, MailtoLink
from pyams_content.component.paragraph.zmi.container import ParagraphContainerCounterBase
from pyams_form.security import ProtectedFormObjectMixin
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.viewlet.toolbar import ToolbarMenuItem
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogAddForm, AdminDialogEditForm
from pyramid.view import view_config
from z3c.form import field

from pyams_content import _


#
# Internal links views
#

@viewlet_config(name='internal-links', context=ILinkContainerTarget, layer=IPyAMSLayer, view=IParagraphContainerTable,
                manager=IParagraphTitleToolbar, permission=VIEW_SYSTEM_PERMISSION, weight=10)
class InternalLinksCounter(ParagraphContainerCounterBase):
    """Internal links count column"""

    weight = 10
    action_class = 'action internal-links nowrap width-40'
    icon_class = 'fa fa-fw fa-external-link-square fa-rotate-90'
    icon_hint = _("Internal links")

    marker_type = 'internal-links'

    @property
    def count(self):
        return len([file for file in IAssociationContainer(self.context).values()
                    if IInternalLink.providedBy(file)])


@viewlet_config(name='add-internal-link.menu', context=ILinkContainerTarget, view=IAssociationsView,
                layer=IPyAMSLayer, manager=IToolbarAddingMenu, weight=50)
class InternalLinkAddMenu(ProtectedFormObjectMixin, ToolbarMenuItem):
    """Internal link add menu"""

    label = _("Add internal link")
    label_css_class = 'fa fa-fw fa-external-link-square fa-rotate-90'

    url = 'add-internal-link.html'
    modal_target = True


@pagelet_config(name='add-internal-link.html', context=ILinkContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class InternalLinkAddForm(AdminDialogAddForm):
    """Internal link add form"""

    legend = _("Add new internal link")
    icon_css_class = 'fa fa-fw fa-external-link-square fa-rotate-90'

    fields = field.Fields(IInternalLink).select('reference', 'title', 'description')
    ajax_handler = 'add-internal-link.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(InternalLinkAddForm, self).updateWidgets(prefix)
        if 'description' in self.widgets:
            self.widgets['description'].widget_css_class = 'textarea'

    def create(self, data):
        return InternalLink()

    def add(self, object):
        IAssociationContainer(self.context).append(object)


@view_config(name='add-internal-link.json', context=ILinkContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class InternalLinkAJAXAddForm(AssociationItemAJAXAddForm, InternalLinkAddForm):
    """Internal link add form, JSON renderer"""

    def get_ajax_output(self, changes):
        output = super(InternalLinkAJAXAddForm, self).get_ajax_output(changes)
        if output:
            output.setdefault('events', []).append(get_json_paragraph_markers_refresh_event(self.context, self.request,
                                                                                            self, InternalLinksCounter))
        return output


@pagelet_config(name='properties.html', context=IInternalLink, layer=IPyAMSLayer, permission=VIEW_SYSTEM_PERMISSION)
class InternalLinkPropertiesEditForm(AdminDialogEditForm):
    """Internal link properties edit form"""

    prefix = 'internallink_properties.'

    legend = _("Edit internal link properties")
    icon_css_class = 'fa fa-fw fa-external-link-square fa-rotate-90'

    fields = field.Fields(IInternalLink).select('reference', 'title', 'description')
    ajax_handler = 'properties.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(InternalLinkPropertiesEditForm, self).updateWidgets(prefix)
        if 'description' in self.widgets:
            self.widgets['description'].widget_css_class = 'textarea'


@view_config(name='properties.json', context=IInternalLink, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class InternalLinkPropertiesAJAXEditForm(AssociationItemAJAXEditForm, InternalLinkPropertiesEditForm):
    """Internal link properties edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        if ('title' in changes.get(IBaseLink, ())) or \
           ('reference' in changes.get(IInternalLink, ())):
            return self.get_associations_table()
        else:
            return super(InternalLinkPropertiesAJAXEditForm, self).get_ajax_output(changes)


#
# External links views
#

@viewlet_config(name='external-links', context=ILinkContainerTarget, layer=IPyAMSLayer, view=IParagraphContainerTable,
                manager=IParagraphTitleToolbar, permission=VIEW_SYSTEM_PERMISSION, weight=11)
class ExternalLinksCounter(ParagraphContainerCounterBase):
    """External links count column"""

    weight = 11
    action_class = 'action external-links nowrap width-40'
    icon_class = 'fa fa-fw fa-external-link'
    icon_hint = _("External links")

    marker_type = 'external-links'

    @property
    def count(self):
        return len([file for file in IAssociationContainer(self.context).values()
                    if IExternalLink.providedBy(file)])


@viewlet_config(name='add-external-link.menu', context=ILinkContainerTarget, view=IAssociationsView,
                layer=IPyAMSLayer, manager=IToolbarAddingMenu, weight=51)
class ExternalLinkAddMenu(ProtectedFormObjectMixin, ToolbarMenuItem):
    """External link add menu"""

    label = _("Add external link")
    label_css_class = 'fa fa-fw fa-external-link'

    url = 'add-external-link.html'
    modal_target = True


@pagelet_config(name='add-external-link.html', context=ILinkContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class ExternalLinkAddForm(AdminDialogAddForm):
    """External link add form"""

    legend = _("Add new external link")
    icon_css_class = 'fa fa-fw fa-external-link'

    fields = field.Fields(IExternalLink).select('url', 'title', 'description', 'language')
    ajax_handler = 'add-external-link.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(ExternalLinkAddForm, self).updateWidgets(prefix)
        if 'description' in self.widgets:
            self.widgets['description'].widget_css_class = 'textarea'

    def create(self, data):
        return ExternalLink()

    def add(self, object):
        IAssociationContainer(self.context).append(object)


@view_config(name='add-external-link.json', context=ILinkContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class ExternalLinkAJAXAddForm(AssociationItemAJAXAddForm, ExternalLinkAddForm):
    """External link add form, JSON renderer"""

    def get_ajax_output(self, changes):
        output = super(ExternalLinkAJAXAddForm, self).get_ajax_output(changes)
        if output:
            output.setdefault('events', []).append(get_json_paragraph_markers_refresh_event(self.context, self.request,
                                                                                            self, ExternalLinksCounter))
        return output


@pagelet_config(name='properties.html', context=IExternalLink, layer=IPyAMSLayer, permission=VIEW_SYSTEM_PERMISSION)
class ExternalLinkPropertiesEditForm(AdminDialogEditForm):
    """External link properties edit form"""

    prefix = 'externallink_properties.'

    legend = _("Edit external link properties")
    icon_css_class = 'fa fa-fw fa-external-link'

    fields = field.Fields(IExternalLink).select('url', 'title', 'description', 'language')
    ajax_handler = 'properties.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(ExternalLinkPropertiesEditForm, self).updateWidgets(prefix)
        if 'description' in self.widgets:
            self.widgets['description'].widget_css_class = 'textarea'


@view_config(name='properties.json', context=IExternalLink, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class ExternalLinkPropertiesAJAXEditForm(AssociationItemAJAXEditForm, ExternalLinkPropertiesEditForm):
    """External link properties edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        if ('title' in changes.get(IBaseLink, ())) or \
           ('url' in changes.get(IExternalLink, ())):
            return self.get_associations_table()
        else:
            return super(ExternalLinkPropertiesAJAXEditForm, self).get_ajax_output(changes)


#
# Mailto links views
#

@viewlet_config(name='mailto-links', context=ILinkContainerTarget, layer=IPyAMSLayer, view=IParagraphContainerTable,
                manager=IParagraphTitleToolbar, permission=VIEW_SYSTEM_PERMISSION, weight=12)
class MailtoLinksCounter(ParagraphContainerCounterBase):
    """Mailto links count column"""

    weight = 12
    action_class = 'action mailto-links nowrap width-40'
    icon_class = 'fa fa-fw fa-envelope-o'
    icon_hint = _("Mailto links")

    marker_type = 'mailto-links'

    @property
    def count(self):
        return len([file for file in IAssociationContainer(self.context).values()
                    if IMailtoLink.providedBy(file)])


@viewlet_config(name='add-mailto-link.menu', context=ILinkContainerTarget, view=IAssociationsView,
                layer=IPyAMSLayer, manager=IToolbarAddingMenu, weight=52)
class MailtoLinkAddMenu(ProtectedFormObjectMixin, ToolbarMenuItem):
    """Mailto link add menu"""

    label = _("Add mailto link")
    label_css_class = 'fa fa-fw fa-envelope-o'

    url = 'add-mailto-link.html'
    modal_target = True


@pagelet_config(name='add-mailto-link.html', context=ILinkContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class MailtoLinkAddForm(AdminDialogAddForm):
    """Mailto link add form"""

    legend = _("Add new mailto link")
    icon_css_class = 'fa fa-fw fa-envelope-o'

    fields = field.Fields(IMailtoLink).select('address', 'address_name', 'title', 'description')
    ajax_handler = 'add-mailto-link.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(MailtoLinkAddForm, self).updateWidgets(prefix)
        if 'description' in self.widgets:
            self.widgets['description'].widget_css_class = 'textarea'

    def create(self, data):
        return MailtoLink()

    def add(self, object):
        IAssociationContainer(self.context).append(object)


@view_config(name='add-mailto-link.json', context=ILinkContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class MailtoLinkAJAXAddForm(AssociationItemAJAXAddForm, MailtoLinkAddForm):
    """Mailto link add form, JSON renderer"""

    def get_ajax_output(self, changes):
        output = super(MailtoLinkAJAXAddForm, self).get_ajax_output(changes)
        if output:
            output.setdefault('events', []).append(get_json_paragraph_markers_refresh_event(self.context, self.request,
                                                                                            self, MailtoLinksCounter))
        return output


@pagelet_config(name='properties.html', context=IMailtoLink, layer=IPyAMSLayer, permission=VIEW_SYSTEM_PERMISSION)
class MailtoLinkPropertiesEditForm(AdminDialogEditForm):
    """Mailto link properties edit form"""

    prefix = 'mailtolink_properties.'

    legend = _("Edit mailto link properties")
    icon_css_class = 'fa fa-fw fa-envelope-o'

    fields = field.Fields(IMailtoLink).select('address', 'address_name', 'title', 'description')
    ajax_handler = 'properties.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(MailtoLinkPropertiesEditForm, self).updateWidgets(prefix)
        if 'description' in self.widgets:
            self.widgets['description'].widget_css_class = 'textarea'


@view_config(name='properties.json', context=IMailtoLink, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class MailtoLinkPropertiesAJAXEditForm(AssociationItemAJAXEditForm, MailtoLinkPropertiesEditForm):
    """Mailto link properties edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        if ('title' in changes.get(IBaseLink, ())) or changes.get(IMailtoLink, ()):
            return self.get_associations_table()
        else:
            return super(MailtoLinkPropertiesAJAXEditForm, self).get_ajax_output(changes)
