#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
from pyams_skin.event import get_json_widget_refresh_event

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.association.zmi.interfaces import IAssociationsParentForm
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget, IParagraphContainer, \
    IParagraphRenderer
from pyams_content.component.paragraph.interfaces.verbatim import IVerbatimParagraph, VERBATIM_PARAGRAPH_TYPE
from pyams_content.component.paragraph.zmi.interfaces import IParagraphInnerEditor, IParagraphContainerView
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_form.interfaces.form import IInnerForm, IEditFormButtons
from pyams_skin.interfaces.viewlet import IToolbarAddingMenu
from pyams_skin.layer import IPyAMSLayer
from pyams_zmi.interfaces import IPropertiesEditForm
from z3c.form.interfaces import INPUT_MODE

# import packages
from pyams_content.component.paragraph.verbatim import VerbatimParagraph
from pyams_content.component.paragraph.zmi import BaseParagraphAJAXAddForm, BaseParagraphAJAXEditForm, \
    BaseParagraphAddMenu, BaseParagraphPropertiesEditForm
from pyams_content.features.renderer.zmi import BaseRenderedContentRenderer
from pyams_content.features.renderer.zmi.widget import RendererFieldWidget
from pyams_pagelet.pagelet import pagelet_config
from pyams_utils.adapter import adapter_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogAddForm
from pyramid.view import view_config
from z3c.form import field, button
from zope.interface import implementer

from pyams_content import _


#
# Verbatim paragraph
#

@viewlet_config(name='add-verbatim-paragraph.menu', context=IParagraphContainerTarget, view=IParagraphContainerView,
                layer=IPyAMSLayer, manager=IToolbarAddingMenu, weight=600)
class VerbatimParagraphAddMenu(BaseParagraphAddMenu):
    """Verbatim paragraph add menu"""

    label = _("Verbatim...")
    label_css_class = 'fa fa-fw fa-quote-right'
    url = 'add-verbatim-paragraph.html'
    paragraph_type = VERBATIM_PARAGRAPH_TYPE


@pagelet_config(name='add-verbatim-paragraph.html', context=IParagraphContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class VerbatimParagraphAddForm(AdminDialogAddForm):
    """Verbatim paragraph add form"""

    legend = _("Add new verbatim paragraph")
    icon_css_class = 'fa fa-fw fa-quote-right'

    fields = field.Fields(IVerbatimParagraph).omit('__parent__', '__name__', 'visible')
    ajax_handler = 'add-verbatim-paragraph.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(VerbatimParagraphAddForm, self).updateWidgets(prefix)
        if 'quote' in self.widgets:
            self.widgets['quote'].widget_css_class = 'textarea'

    def create(self, data):
        return VerbatimParagraph()

    def add(self, object):
        IParagraphContainer(self.context).append(object)


@view_config(name='add-verbatim-paragraph.json', context=IParagraphContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class VerbatimParagraphAJAXAddForm(BaseParagraphAJAXAddForm, VerbatimParagraphAddForm):
    """Verbatim paragraph add form, JSON renderer"""


@pagelet_config(name='properties.html', context=IVerbatimParagraph, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class VerbatimParagraphPropertiesEditForm(BaseParagraphPropertiesEditForm):
    """Verbatim paragraph properties edit form"""

    prefix = 'verbatim_properties.'

    legend = _("Edit verbatim paragraph properties")
    icon_css_class = 'fa fa-fw fa-quote-right'

    fields = field.Fields(IVerbatimParagraph).omit('__parent__', '__name__', 'visible')
    fields['renderer'].widgetFactory = RendererFieldWidget

    ajax_handler = 'properties.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(VerbatimParagraphPropertiesEditForm, self).updateWidgets(prefix)
        if 'quote' in self.widgets:
            self.widgets['quote'].widget_css_class = 'textarea'


@view_config(name='properties.json', context=IVerbatimParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class VerbatimParagraphPropertiesAJAXEditForm(BaseParagraphAJAXEditForm, VerbatimParagraphPropertiesEditForm):
    """Verbatim paragraph properties edit form, JSON renderer"""


@adapter_config(context=(IVerbatimParagraph, IPyAMSLayer), provides=IParagraphInnerEditor)
@implementer(IInnerForm, IPropertiesEditForm, IAssociationsParentForm)
class VerbatimParagraphInnerEditForm(VerbatimParagraphPropertiesEditForm):
    """Verbatim paragraph inner edit form"""

    legend = None
    ajax_handler = 'inner-properties.json'

    @property
    def buttons(self):
        if self.mode == INPUT_MODE:
            return button.Buttons(IEditFormButtons)
        else:
            return button.Buttons()


@view_config(name='inner-properties.json', context=IVerbatimParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class VerbatimParagraphInnerAJAXEditForm(BaseParagraphAJAXEditForm, VerbatimParagraphInnerEditForm):
    """Verbatim paragraph inner edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        output = super(VerbatimParagraphInnerAJAXEditForm, self).get_ajax_output(changes)
        updated = changes.get(IVerbatimParagraph, ())
        if 'renderer' in updated:
            output.setdefault('events', []).append(
                get_json_widget_refresh_event(self.context, self.request, VerbatimParagraphInnerEditForm, 'renderer'))
        return output


#
# Verbatim paragraph renderer
#

@adapter_config(context=(IVerbatimParagraph, IPyAMSLayer), provides=IParagraphRenderer)
class VerbatimParagraphRenderer(BaseRenderedContentRenderer):
    """Verbatim paragraph renderer"""
