#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.association.zmi.interfaces import IAssociationsParentForm
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget, IParagraphContainer, \
    IParagraphRenderer
from pyams_content.component.paragraph.interfaces.video import IVideoParagraph, VIDEO_PARAGRAPH_TYPE
from pyams_content.component.paragraph.zmi.interfaces import IParagraphInnerEditor, IParagraphContainerView
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_form.interfaces.form import IInnerForm, IEditFormButtons
from pyams_skin.interfaces.viewlet import IToolbarAddingMenu
from pyams_skin.layer import IPyAMSLayer
from pyams_zmi.interfaces import IPropertiesEditForm
from transaction.interfaces import ITransactionManager
from z3c.form.interfaces import INPUT_MODE

# import packages
from pyams_content.component.association.zmi import AssociationsTable
from pyams_content.component.paragraph.video import VideoParagraph
from pyams_content.component.paragraph.zmi import BaseParagraphAJAXAddForm, BaseParagraphAJAXEditForm, \
    BaseParagraphAddMenu, BaseParagraphPropertiesEditForm
from pyams_content.features.renderer.zmi import BaseRenderedContentRenderer
from pyams_content.features.renderer.zmi.widget import RendererFieldWidget
from pyams_form.group import NamedWidgetsGroup
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.event import get_json_form_refresh_event, get_json_widget_refresh_event
from pyams_utils.adapter import adapter_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogAddForm
from pyramid.view import view_config
from z3c.form import field, button
from zope.interface import implementer

from pyams_content import _


@viewlet_config(name='add-video-paragraph.menu', context=IParagraphContainerTarget, view=IParagraphContainerView,
                layer=IPyAMSLayer, manager=IToolbarAddingMenu, weight=70)
class VideoParagraphAddMenu(BaseParagraphAddMenu):
    """Video paragraph add menu"""

    label = _("Video paragraph...")
    label_css_class = 'fa fa-fw fa-film'
    url = 'add-video-paragraph.html'
    paragraph_type = VIDEO_PARAGRAPH_TYPE


@pagelet_config(name='add-video-paragraph.html', context=IParagraphContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class VideoParagraphAddForm(AdminDialogAddForm):
    """Video paragraph add form"""

    legend = _("Add new video paragraph")
    dialog_class = 'modal-large'
    icon_css_class = 'fa fa-fw fa-film'

    fields = field.Fields(IVideoParagraph).omit('__parent__', '__name__', 'visible')
    ajax_handler = 'add-video-paragraph.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(VideoParagraphAddForm, self).updateWidgets(prefix)
        if 'description' in self.widgets:
            self.widgets['description'].widget_css_class = 'textarea'
        if 'body' in self.widgets:
            self.widgets['body'].label = ''
            self.add_group(NamedWidgetsGroup(self, 'body_group', self.widgets, ('body',),
                                             bordered=False,
                                             legend=_("HTML content"),
                                             css_class='inner switcher padding-right-10 no-y-padding pull-left',
                                             switch=True,
                                             hide_if_empty=True))
            self.add_group(NamedWidgetsGroup(self, 'data_group', self.widgets,
                                             ('description', 'author', 'data', 'renderer'),
                                             bordered=False))

    def create(self, data):
        return VideoParagraph()

    def add(self, object):
        IParagraphContainer(self.context).append(object)


@view_config(name='add-video-paragraph.json', context=IParagraphContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class VideoParagraphAJAXAddForm(BaseParagraphAJAXAddForm, VideoParagraphAddForm):
    """Video paragraph add form, JSON renderer"""


@pagelet_config(name='properties.html', context=IVideoParagraph, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class VideoParagraphPropertiesEditForm(BaseParagraphPropertiesEditForm):
    """Video paragraph properties edit form"""

    prefix = 'video_properties.'

    legend = _("Edit video properties")
    dialog_class = 'modal-large'
    icon_css_class = 'fa fa-fw fa-film'

    fields = field.Fields(IVideoParagraph).omit('__parent__', '__name__', 'visible')
    fields['renderer'].widgetFactory = RendererFieldWidget

    ajax_handler = 'properties.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(VideoParagraphPropertiesEditForm, self).updateWidgets(prefix)
        if 'description' in self.widgets:
            self.widgets['description'].widget_css_class = 'textarea'
        if 'body' in self.widgets:
            self.widgets['body'].label = ''
            self.add_group(NamedWidgetsGroup(self, 'body_group', self.widgets, ('body',),
                                             bordered=False,
                                             fieldset_class='margin-top-10 padding-y-5',
                                             legend=_("HTML content"),
                                             css_class='inner switcher padding-right-10 no-y-padding pull-left',
                                             switch=True,
                                             hide_if_empty=True))
            self.add_group(NamedWidgetsGroup(self, 'data_group', self.widgets,
                                             ('description', 'author', 'data', 'renderer'),
                                             bordered=False))


@view_config(name='properties.json', context=IVideoParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class VideoParagraphPropertiesAJAXEditForm(BaseParagraphAJAXEditForm, VideoParagraphPropertiesEditForm):
    """Video paragraph properties edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        output = super(VideoParagraphPropertiesAJAXEditForm, self).get_ajax_output(changes)
        if 'body' in changes.get(IVideoParagraph, ()):
            associations_table = AssociationsTable(self.context, self.request)
            associations_table.update()
            output.setdefault('events', []).append({
                'event': 'PyAMS_content.changed_item',
                'options': {'handler': 'PyAMS_content.associations.refreshAssociations',
                            'object_name': associations_table.id,
                            'table': associations_table.render()}})
        return output


@adapter_config(context=(IVideoParagraph, IPyAMSLayer), provides=IParagraphInnerEditor)
@implementer(IInnerForm, IPropertiesEditForm, IAssociationsParentForm)
class VideoParagraphPropertiesInnerEditForm(VideoParagraphPropertiesEditForm):
    """Video paragraph properties inner edit form"""

    legend = None
    ajax_handler = 'inner-properties.json'

    @property
    def buttons(self):
        if self.mode == INPUT_MODE:
            return button.Buttons(IEditFormButtons)
        else:
            return button.Buttons()


@view_config(name='inner-properties.json', context=IVideoParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class VideoParagraphPropertiesInnerAJAXEditForm(BaseParagraphAJAXEditForm, VideoParagraphPropertiesInnerEditForm):
    """Video paragraph properties inner deit form, JSON renderer"""

    def get_ajax_output(self, changes):
        output = super(VideoParagraphPropertiesInnerAJAXEditForm, self).get_ajax_output(changes)
        updated = changes.get(IVideoParagraph, ())
        if 'body' in updated:
            associations_table = AssociationsTable(self.context, self.request)
            associations_table.update()
            output.setdefault('events', []).append({
                'event': 'PyAMS_content.changed_item',
                'options': {'handler': 'PyAMS_content.associations.refreshAssociations',
                            'object_name': associations_table.id,
                            'table': associations_table.render()}
            })
        if 'data' in updated:
            # we have to commit transaction to be able to handle blobs...
            ITransactionManager(self.context).get().commit()
            output.setdefault('events', []).append(
                get_json_form_refresh_event(self.context, self.request, VideoParagraphPropertiesInnerEditForm))
        elif 'renderer' in updated:
            output.setdefault('events', []).append(
                get_json_widget_refresh_event(self.context, self.request,
                                              VideoParagraphPropertiesInnerEditForm, 'renderer'))
        return output


#
# Video paragraph renderer
#

@adapter_config(context=(IVideoParagraph, IPyAMSLayer), provides=IParagraphRenderer)
class VideoParagraphRenderer(BaseRenderedContentRenderer):
    """Video paragraph renderer"""
