#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.theme.interfaces import IPortletThemesSettings, IPortletThemesSettingsTarget, \
    PORTLET_SETTINGS_THEMES_KEY
from zope.annotation.interfaces import IAnnotations
from zope.traversing.interfaces import ITraversable

# import packages
from persistent import Persistent
from pyams_utils.adapter import adapter_config, ContextAdapter
from zope.container.contained import Contained
from zope.interface import implementer
from zope.location import locate


@implementer(IPortletThemesSettings)
class PortletThemesSettings(Persistent, Contained):
    """Portlet themes settings"""


@adapter_config(context=IPortletThemesSettingsTarget, provides=IPortletThemesSettings)
def portlet_themes_settings_factory(context):
    """Portlet themes settings adapter"""
    annotations = IAnnotations(context)
    settings = annotations.get(PORTLET_SETTINGS_THEMES_KEY)
    if settings is None:
        settings = annotations[PORTLET_SETTINGS_THEMES_KEY] = PortletThemesSettings()
        locate(settings, context, '++themes++')
    return settings


@adapter_config(name='themes', context=IPortletThemesSettingsTarget, provides=ITraversable)
class ThemesPortletsSettingsTraverser(ContextAdapter):
    """++themes++ portlet settings adapter"""

    def traverse(self, name, furtherpath=None):
        return IPortletThemesSettings(self.context)
