#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.features.alert.interfaces import IAlertContainer, IAlertItem, IAlertTarget, ALERT_CONTAINER_KEY
from zope.annotation.interfaces import IAnnotations
from zope.location.interfaces import ISublocations
from zope.traversing.interfaces import ITraversable

# import packages
from pyams_catalog.utils import index_object
from pyams_utils.adapter import adapter_config, ContextAdapter
from pyams_utils.registry import get_current_registry
from zope.container.ordered import OrderedContainer
from zope.interface import implementer
from zope.lifecycleevent import ObjectCreatedEvent
from zope.location import locate


@implementer(IAlertContainer)
class AlertContainer(OrderedContainer):
    """Alert container persistent class"""

    last_id = 1

    def append(self, value, notify=True):
        key = str(self.last_id)
        if not notify:
            # pre-locate alert item to avoid multiple notifications
            locate(value, self, key)
        self[key] = value
        self.last_id += 1
        if not notify:
            # make sure that alert item is correctly indexed
            index_object(value)

    def get_visible_items(self):
        return filter(lambda x: IAlertItem(x).visible, self.values())


@adapter_config(context=IAlertTarget, provides=IAlertContainer)
def alert_container_factory(target):
    """Alert container factory"""
    annotations = IAnnotations(target)
    container = annotations.get(ALERT_CONTAINER_KEY)
    if container is None:
        container = annotations[ALERT_CONTAINER_KEY] = AlertContainer()
        get_current_registry().notify(ObjectCreatedEvent(container))
        locate(container, target, '++alert++')
    return container


@adapter_config(name='alert', context=IAlertTarget, provides=ITraversable)
class AlertContainerNamespace(ContextAdapter):
    """Alert container ++alert++ namespace"""

    def traverse(self, name, furtherpath=None):
        return IAlertContainer(self.context)


@adapter_config(name='alerts', context=IAlertTarget, provides=ISublocations)
class AlertContainerSublocations(ContextAdapter):
    """Alert container sub-locations adapter"""

    def sublocations(self):
        return IAlertContainer(self.context).values()
