#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.features.alert.interfaces import IAlertTarget, IAlertItem, IAlertContainer
from pyams_content.interfaces import MANAGE_SITE_ROOT_PERMISSION
from pyams_skin.interfaces.viewlet import IWidgetTitleViewletManager
from pyams_skin.layer import IPyAMSLayer

# import packages
from pyams_content.features.alert import AlertItem
from pyams_content.features.alert.zmi.container import AlertContainerView, AlertContainerTable
from pyams_form.form import AJAXAddForm, AJAXEditForm
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.event import get_json_table_row_refresh_event
from pyams_skin.viewlet.toolbar import ToolbarAction
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogAddForm, AdminDialogEditForm
from pyramid.view import view_config
from z3c.form import field

from pyams_content import _


@viewlet_config(name='add-alert.action', context=IAlertTarget, layer=IPyAMSLayer, view=AlertContainerView,
                manager=IWidgetTitleViewletManager, permission=MANAGE_SITE_ROOT_PERMISSION, weight=1)
class AlertItemAddAction(ToolbarAction):
    """Alert item add action"""

    label = _("Add alert")
    label_css_class = 'fa fa-fw fa-plus'
    url = 'add-alert.html'
    modal_target = True


@pagelet_config(name='add-alert.html', context=IAlertTarget, layer=IPyAMSLayer, permission=MANAGE_SITE_ROOT_PERMISSION)
class AlertItemAddForm(AdminDialogAddForm):
    """Alert item add form"""

    legend = _("Add new alert")
    icon_css_class = 'fa fa-fw fa-exclamation-triangle'

    fields = field.Fields(IAlertItem).omit('__parent__', '__name__', 'visible')
    ajax_handler = 'add-alert.json'
    edit_permission = MANAGE_SITE_ROOT_PERMISSION

    def create(self, data):
        return AlertItem()

    def add(self, object):
        IAlertContainer(self.context).append(object)

    def nextURL(self):
        return absolute_url(self.context, self.request, 'alerts.html')


@view_config(name='add-alert.json', context=IAlertTarget, request_type=IPyAMSLayer,
             permission=MANAGE_SITE_ROOT_PERMISSION, renderer='json', xhr=True)
class AlertItemAJAXAddForm(AJAXAddForm, AlertItemAddForm):
    """Alert item add form, JSON renderer"""


@pagelet_config(name='properties.html', context=IAlertItem, layer=IPyAMSLayer, permission=MANAGE_SITE_ROOT_PERMISSION)
class AlertItemPropertiesEditForm(AdminDialogEditForm):
    """Alert item properties edit form"""

    prefix = 'alert_properties.'

    legend = _("Edit alert properties")
    icon_css_class = 'fa fa-fw fa-exclamation-triangle'

    fields = field.Fields(IAlertItem).omit('__parent__', '__name__', 'visible')
    ajax_handler = 'properties.json'
    edit_permission = MANAGE_SITE_ROOT_PERMISSION


@view_config(name='properties.json', context=IAlertItem, request_type=IPyAMSLayer,
             permission=MANAGE_SITE_ROOT_PERMISSION, renderer='json', xhr=True)
class AlertItemPropertiesAJAXEditForm(AJAXEditForm, AlertItemPropertiesEditForm):
    """Alert item properties edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        output = super(AlertItemPropertiesAJAXEditForm, self).get_ajax_output(changes)
        updated = changes.get(IAlertItem, ())
        if updated:
            target = get_parent(self.context, IAlertTarget)
            output.setdefault('events', []).append(
                get_json_table_row_refresh_event(target, self.request, AlertContainerTable, self.context))
        return output
