#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.features.renderer.interfaces import IRenderedContent, IContentRenderer, IRendererSettings

# import packages
from pyams_utils.adapter import adapter_config
from pyams_utils.request import check_request
from zope.interface import implementer


@implementer(IRenderedContent)
class RenderedContentMixin(object):
    """Renderer mixin interface"""

    renderer = None

    def get_renderer(self, request=None):
        if request is None:
            request = check_request()
        return request.registry.queryMultiAdapter((self, request), IContentRenderer, name=self.renderer)


@adapter_config(context=IRenderedContent, provides=IContentRenderer)
def rendered_content_renderer_factory(context):
    """Rendered content renderer factory"""
    return context.get_renderer()


@adapter_config(context=IRenderedContent, provides=IRendererSettings)
def rendered_content_renderer_settings_factory(context):
    """Rendered content renderer settings factory"""
    renderer = IContentRenderer(context)
    if renderer.settings_interface is None:
        return None
    return renderer.settings_interface(context)
