#
# Copyright (c) 2008-2016 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.illustration import IIllustrationTarget
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget
from pyams_content.component.theme.interfaces import IThemesTarget
from pyams_content.features.preview.interfaces import IPreviewTarget
from pyams_content.features.review.interfaces import IReviewTarget
from pyams_content.shared.blog.interfaces import IWfBlogPost, BLOG_CONTENT_TYPE, BLOG_CONTENT_NAME, IBlogPost, \
    IWfBlogPostFactory
from pyams_workflow.interfaces import IWorkflow, IWorkflowVersions, IWorkflowState

# import packages
from pyams_content.shared.common import WfSharedContent, register_content_type, SharedContent, IWfSharedContentFactory
from pyams_utils.adapter import adapter_config
from zope.interface import implementer, provider


@implementer(IWfBlogPost, IParagraphContainerTarget, IThemesTarget,
             IIllustrationTarget, IPreviewTarget, IReviewTarget)
class WfBlogPost(WfSharedContent):
    """Base blog post"""

    content_type = BLOG_CONTENT_TYPE
    content_name = BLOG_CONTENT_NAME

register_content_type(WfBlogPost)


@provider(IWfBlogPostFactory)
@implementer(IBlogPost)
class BlogPost(SharedContent):
    """Worfklow managed blog post class"""

    def is_deletable(self):
        workflow = IWorkflow(self)
        for version in IWorkflowVersions(self).get_versions():
            if IWorkflowState(version).state != workflow.initial_state:
                return False
        return True


@adapter_config(context=IWfBlogPostFactory, provides=IWfSharedContentFactory)
def blog_post_content_factory(context):
    return WfBlogPost
