#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
from pyramid.events import subscriber
from z3c.form.interfaces import IDataExtractedEvent, HIDDEN_MODE

from pyams_skin.event import get_json_widget_refresh_event
from pyams_utils.url import generate_url

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION, IBaseContent
from pyams_content.shared.common.interfaces import IWfSharedContent
from pyams_form.interfaces.form import IWidgetForm
from pyams_skin.interfaces import IInnerPage, IPageHeader
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.interfaces import VIEW_SYSTEM_PERMISSION
from pyams_zmi.interfaces import IPropertiesEditForm
from pyams_zmi.interfaces.menu import IContentManagementMenu, IPropertiesMenu
from pyams_zmi.layer import IAdminLayer

# import packages
from pyams_content.shared.common.zmi import WfSharedContentHeaderAdapter
from pyams_form.form import AJAXEditForm
from pyams_i18n.widget import I18nSEOTextLineFieldWidget
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.viewlet.menu import MenuItem
from pyams_utils.adapter import adapter_config
from pyams_viewlet.manager import viewletmanager_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm
from pyramid.view import view_config
from z3c.form import field
from zope.interface import implementer

from pyams_content import _


#
# Content properties
#

@viewlet_config(name='properties.menu', context=IWfSharedContent, layer=IAdminLayer,
                manager=IContentManagementMenu, permission=VIEW_SYSTEM_PERMISSION, weight=10)
@viewletmanager_config(name='properties.menu', layer=IAdminLayer, provides=IPropertiesMenu)
@implementer(IPropertiesMenu)
class SharedContentCompositionMenu(MenuItem):
    """Shared content properties menu"""

    label = _("Composition")
    icon_class = 'fa-dropbox'
    url = '#summary.html'


@viewlet_config(name='properties.submenu', context=IWfSharedContent, layer=IAdminLayer,
                manager=IPropertiesMenu, permission=MANAGE_CONTENT_PERMISSION, weight=10)
class SharedContentPropertiesMenu(MenuItem):
    """Shared content properties menu"""

    label = _("Properties")
    icon_class = 'fa-edit'
    url = '#properties.html'


@pagelet_config(name='properties.html', context=IWfSharedContent, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
@implementer(IPropertiesEditForm, IWidgetForm, IInnerPage)
class SharedContentPropertiesEditForm(AdminEditForm):
    """Shared content properties edit form"""

    legend = _("Content properties")

    fields = field.Fields(IWfSharedContent).select('title', 'short_name', 'content_url',
                                                   'description', 'notepad')
    fields['title'].widgetFactory = I18nSEOTextLineFieldWidget

    ajax_handler = 'properties.json'

    def updateWidgets(self, prefix=None):
        super(SharedContentPropertiesEditForm, self).updateWidgets(prefix)
        if 'short_name' in self.widgets:
            self.widgets['short_name'].set_mode(HIDDEN_MODE)
        if 'description' in self.widgets:
            self.widgets['description'].widget_css_class = 'textarea'
        if 'notepad' in self.widgets:
            self.widgets['notepad'].widget_css_class = 'textarea'


@subscriber(IDataExtractedEvent, form_selector=SharedContentPropertiesEditForm)
def handle_content_properties_data_extraction(event):
    """Automatically set short_name as title"""
    data = event.data
    data['short_name'] = data['title'].copy()
    data['content_url'] = generate_url(data['content_url'])


@view_config(name='properties.json', context=IWfSharedContent, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class SharedContentPropertiesAJAXEditForm(AJAXEditForm, SharedContentPropertiesEditForm):
    """Shared content properties edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        updated = changes.get(IBaseContent, ())
        if 'title' in updated:
            return {
                'status': 'reload',
                'message': self.request.localizer.translate(self.successMessage)
            }
        else:
            output = super(SharedContentPropertiesAJAXEditForm, self).get_ajax_output(changes)
            updated = changes.get(IWfSharedContent, ())
            if 'content_url' in updated:
                output.setdefault('events', []).append(
                    get_json_widget_refresh_event(self.context, self.request,
                                                  SharedContentPropertiesEditForm, 'content_url'))
            return output


@adapter_config(context=(IWfSharedContent, IAdminLayer, SharedContentPropertiesEditForm), provides=IPageHeader)
class SharedContentPropertiesHeaderAdapter(WfSharedContentHeaderAdapter):
    """Shared content properties header adapter"""

    icon_class = 'fa fa-fw fa-edit'
