#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.shared.site.interfaces import IContentLink
from pyams_workflow.interfaces import IWorkflow, IWorkflowVersion, IWorkflowVersions, IWorkflowPublicationInfo, \
    IWorkflowState

# import packages
from persistent import Persistent
from pyams_sequence.utility import get_reference_target
from pyams_utils.adapter import adapter_config
from zope.container.contained import Contained
from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty


@implementer(IContentLink)
class ContentLink(Persistent, Contained):
    """Content link persistent class

    A 'content link' is a link to another content, which may be stored anywhere (same site,
    another site or in any shared tool).
    """

    reference = FieldProperty(IContentLink['reference'])
    alt_title = FieldProperty(IContentLink['alt_title'])
    visible = FieldProperty(IContentLink['visible'])

    def is_deletable(self):
        return True

    def get_target(self):
        target = get_reference_target(self.reference)
        if IWorkflowVersion.providedBy(target):
            workflow = IWorkflow(target, None)
            if workflow is not None:
                versions = IWorkflowVersions(target).get_versions(workflow.published_states, sort=True)
                if not versions:
                    versions = IWorkflowVersions(target).get_last_versions()
                if versions:
                    target = versions[-1]
        return target


@adapter_config(context=IContentLink, provides=IWorkflow)
def content_link_workflow_info(context):
    """Content link workflow info"""
    target = context.get_target()
    if target is not None:
        return IWorkflow(target, None)


@adapter_config(context=IContentLink, provides=IWorkflowState)
def content_link_state_info(context):
    """Content link workflow state info"""
    target = context.get_target()
    if target is not None:
        return IWorkflowState(target, None)


@adapter_config(context=IContentLink, provides=IWorkflowPublicationInfo)
def content_link_publication_info(context):
    """Content link publication info"""
    target = context.get_target()
    if target is not None:
        return IWorkflowPublicationInfo(target, None)
